import os
from glob import glob
import re
#from aeri_tools.io.dmv.housekeeping import get_all_housekeeping

import electronic_checks
import global_checks
import radiometric_checks
import scene_checks
import state_checks
import thermal_checks

levels = [
    global_checks.CheckList(),
    scene_checks.CheckList(),
    state_checks.CheckList(), 
    electronic_checks.CheckList(),
    radiometric_checks.CheckList(),
    thermal_checks.CheckList()
]

def read_frame(cxs_file, sum_file):
    return get_all_housekeeping(cxs_file).combine_first(
        get_all_housekeeping(sum_file))

def check_frame(frame, parameters):
    frame['qc_percent'] = 0
    frame['qc_notes'] = None
    for level in levels:
        level.set_params(parameters)
        frame = level.compute(frame)
    return frame

def update_all(ftp_dir, parameters=None):
    cxs_files = glob(os.path.join(ftp_dir,'AE*','*B1.CXS'))
    for qc_file, cxs_file, sum_file in files_to_update(cxs_files):
        hk = read_frame(cxs_file, sum_file)
        if parameters is None:
            parameters = {}
        check_frame(frame, parameters)

def files_to_update(cxs_files):
    for cxs_file in cxs_files:
        possible_sum = os.path.join(os.path.dirname(cxs_file), cxs_file.replace('B1.CXS','.SUM'))
        possible_qc = os.path.join(os.path.dirname(cxs_file), cxs_file.replace('B1.CXS','.qc'))

        if os.path.isfile(possible_sum):
            sum_file = possible_sum

            if os.path.isfile(possible_qc):
                qc_file = possible_qc
                if max(os.path.getmtime(sum_file), os.path.getmtime(cxs_file)) > os.path.getmtime(qc_file):
                    yield (qc_file, cxs_file, sum_file)
            else:
                yield (qc_file, cxs_file, sum_file)

if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('ftp')

    args = parser.parse_args()

    update_all(args.ftp)
